// Copyright 2015 Concrete Software. All rights reserved.
// Water shader for "Blackbeard's Cove" venue.
// Water shader works by using a single bump map for the water,
// at three different scales, moving at three different speeds
// and directions. The combination of these bump maps is used for
// the actual bump map calculation.

uniform mediump mat3 cs_NormalMatrix;
uniform sampler2D pba_BumpTexture;

// space between samples in bump texture (1/512)
#define DELTA 0.001953125
// Adjusts the height of bumps. This is equal to 1/(amount of change needed in the sample to be represented as a 45 degree angle.) The old default value was 2.
uniform mediump float pba_BumpHeight;

uniform sampler2D texture;
// Direction of the main light in eye coordinates. Must be normalized.
uniform mediump vec3 pba_KeyLightPosition;
// Shininess amount. 128 is very shiny. 1 is not so shiny.
uniform mediump float pba_KeyShininess;
// values for "Rim" light on side to bring out shape
uniform mediump vec3 pba_RimLightPosition;
//uniform lowp float pba_RimShininess;
// color (and intensity) for the diffuse light
uniform mediump vec3 pba_diffuseColor;
// color (and intensity) for the ambient light
uniform mediump vec3 pba_ambientColor;
// color (and intensity) for the specular component of the light
uniform mediump vec3 pba_specularColor;

// Shininess amount. 128 is very shiny. 1 is not so shiny.
uniform mediump float pba_RimShininess;
// color (and intensity) for the specular component of the light coming from the second light
uniform mediump vec3 pba_RimSpecularColor;

// normals passed through from vertex shader
varying mediump vec3 var_Normal;
// vertex in eye space (no projection)
varying mediump vec3 var_TransformedVertex;
// Tex coordinate
varying mediump vec2 var_TexCoord;
// bump map coordinates (each at different positions and scales)
varying highp vec2 var_BumpTexCoord1;
varying highp vec2 var_BumpTexCoord2;
varying highp vec2 var_BumpTexCoord3;

// water is just a simple plane, so we don't pass these in like we do with balls.
// tangent and bitangent are vectors that lie in/define the plane the water.
const mediump vec3 tangent = vec3(1.0, 0.0, 0.0);
const mediump vec3 bitangent = vec3(0.0, 0.0, 1.0);

mediump float getHeight(highp vec2 coordinate);

// These should match the scales used in water.vs. Except that they don't, because I forgot to change them once and we liked how it looked.
const highp float scale1 = 1.25;
const highp float scale2 = 0.75;
const highp float scale3 = 0.25;

void main()
{
    // Height is the sum of samples from all three bump maps
    mediump float height = getHeight(var_BumpTexCoord1) + getHeight(var_BumpTexCoord2) + getHeight(var_BumpTexCoord3);
    mediump float heightSampleX = getHeight(vec2(var_BumpTexCoord1.x+DELTA*scale1, var_BumpTexCoord1.y)) + getHeight(vec2(var_BumpTexCoord2.x+DELTA*scale2, var_BumpTexCoord2.y)) + getHeight(vec2(var_BumpTexCoord3.x+DELTA*scale3, var_BumpTexCoord3.y));
    mediump float heightSampleY = getHeight(vec2(var_BumpTexCoord1.x, var_BumpTexCoord1.y+DELTA*scale1)) + getHeight(vec2(var_BumpTexCoord2.x, var_BumpTexCoord2.y+DELTA*scale2)) + getHeight(vec2(var_BumpTexCoord3.x, var_BumpTexCoord3.y+DELTA*scale3));
    mediump float heightDX = (height-heightSampleX)*pba_BumpHeight;
    mediump float heightDY = (height-heightSampleY)*pba_BumpHeight;
    // 0, 0, 1 is just a normal smooth surface.
    mediump vec3 normal = normalize(vec3(heightDX,heightDY,max(1.-heightDX*heightDX-heightDY*heightDY, 0.)));
    
    
    // transform normal back to object space
    normal = mat3(tangent, bitangent, var_Normal) * normal;
    
	mediump vec3 normalizedNormal = normalize(cs_NormalMatrix * normal);
        
	mediump float diffuseIntensity = max(dot(normalizedNormal, pba_KeyLightPosition), 0.0);
    diffuseIntensity += max(dot(normalizedNormal, pba_RimLightPosition), 0.0);
	
    // key specular component
	mediump vec3 eyePosition = normalize(var_TransformedVertex);
	mediump vec3 halfLight = normalize(pba_KeyLightPosition - eyePosition);
	mediump float normalDotHalf = max(dot(normalizedNormal, halfLight), 0.);
	mediump float specularComponent = pow(normalDotHalf, pba_KeyShininess);
    
    // rim specular component
	halfLight = normalize(pba_RimLightPosition - eyePosition);
	normalDotHalf = max(dot(normalizedNormal, halfLight), 0.);
	mediump float rimSpecularComponent = pow(normalDotHalf, pba_RimShininess);

    // basic color is lighting and texture
    mediump vec4 basicColor = vec4(pba_ambientColor + pba_diffuseColor * diffuseIntensity, 1.0) * texture2D(texture, var_TexCoord);
    // add in specular component to get the final result. Water is more opaque where there is a specular component.
	gl_FragColor = basicColor + vec4(specularComponent * pba_specularColor + rimSpecularComponent * pba_RimSpecularColor, (specularComponent + rimSpecularComponent));
}

mediump float getHeight(highp vec2 coordinate)
{
    // bump map is single channel only, so taking any channel (not just red) would work.
    return texture2D(pba_BumpTexture, coordinate).r;
}
